@@ -1,4 +1,4 @@
-// Copyright © 2011, 2012 Richard Kettlewell.
+// Copyright © 2011, 2012, 2015 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,16 +17,12 @@
 #include "Utils.h"
 #include "Command.h"
 #include <cassert>
+#include "BulkRemove.h"
 
-void BulkRemove(const std::string &path) {
+void BulkRemove::initialize(const std::string &path) {
   // Invoking rm makes more sense than re-implementing it.
-  std::vector<std::string> cmd;
-  cmd.push_back("rm");
-  cmd.push_back("-rf");
-  cmd.push_back(path);
-  Subprocess sp(cmd);
+  std::vector<std::string> cmd = { "rm", "-rf", path };
+  setCommand(cmd);
   if(command.verbose)
-    sp.report();
-  if(command.act)
-    sp.runAndWait();
+    report();
 }
