@@ -1,5 +1,5 @@
 // -*-C++-*-
-// Copyright © 2011, 2012, 2014 Richard Kettlewell.
+// Copyright © 2011, 2012, 2014, 2015 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,18 +23,17 @@
 #include <string>
 #include <vector>
 #include <climits>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
 
 #include "Defaults.h"
 #include "Date.h"
-#include "Regexp.h"
 
 class Store;
 class Device;
 class Host;
 class Volume;
+class Database;
+class Backup;
 
 /** @brief Base for Volume, Host and Conf
  *
@@ -49,41 +48,36 @@ public:
    *
    * @see @ref Defaults.h
    */
-  ConfBase(): maxAge(DEFAULT_MAX_AGE),
-              minBackups(DEFAULT_MIN_BACKUPS),
-              pruneAge(DEFAULT_PRUNE_AGE),
-              rsyncTimeout(0),
-              hookTimeout(0),
-              devicePattern("*") {}
+  ConfBase() = default;
+
+  ConfBase(const ConfBase &) = delete;
+  ConfBase& operator=(const ConfBase &) = delete;
 
   /** @brief Constructor that inherits from a parent
    * @param parent Parent container
    */
   ConfBase(ConfBase *parent): maxAge(parent->maxAge),
-                              minBackups(parent->minBackups),
-                              pruneAge(parent->pruneAge),
+                              prunePolicy(parent->prunePolicy),
+                              pruneParameters(parent->pruneParameters),
                               preBackup(parent->preBackup),
                               postBackup(parent->postBackup),
                               rsyncTimeout(parent->rsyncTimeout),
+                              sshTimeout(parent->sshTimeout),
                               hookTimeout(parent->hookTimeout),
                               devicePattern(parent->devicePattern) {}
 
-  virtual ~ConfBase();
+  virtual ~ConfBase() = default;
 
   /** @brief Maximum comfortable age of most recent backup
    *
    * Corresponds to @c max-age. */
-  int maxAge;
+  int maxAge = DEFAULT_MAX_AGE;
 
-  /** @brief Minimum comfortable number of backups
-   *
-   * Corresponds to @c min-backups */
-  int minBackups;
+  /** @brief Name of pruning policy */
+  std::string prunePolicy = DEFAULT_PRUNE_POLICY;
 
-  /** @brief Age at which to prune backups
-   *
-   * Corresponds to @c prune-age */
-  int pruneAge;
+  /** @brief Pruning policy parameters */
+  std::map<std::string, std::string> pruneParameters;
 
   /** @brief Pre-backup hook */
   std::vector<std::string> preBackup;
@@ -92,19 +86,32 @@ public:
   std::vector<std::string> postBackup;
 
   /** @brief rsync timeout */
-  int rsyncTimeout;
+  int rsyncTimeout = 0;
+
+  /** @brief Timeout to pass to SSH */
+  int sshTimeout = DEFAULT_SSH_TIMEOUT;
 
   /** @brief hook timeout */
-  int hookTimeout;
+  int hookTimeout = 0;
 
   /** @brief Device pattern to be used */
-  std::string devicePattern;
+  std::string devicePattern = "*";
 
   /** @brief Write this node to a stream
    * @param os Output stream
    * @param step Indent depth
+   * @param verbose Include informative annotations
    */
-  virtual void write(std::ostream &os, int step = 0) const;
+  virtual void write(std::ostream &os, int step, bool verbose)
+    const;
+
+  /** @brief Return the parent of this configuration node
+   * @return Parent node or null pointer
+   */
+  virtual ConfBase *getParent() const = 0;
+
+  /** @brief Return a description of this node */
+  virtual std::string what() const = 0;
 
 protected:
   /** @brief Quote a string for use in the config file
@@ -125,45 +132,63 @@ protected:
    */
   static std::string indent(int step);
 
+  /** @brief Type for description functions
+   * @param os Output stream
+   * @param description Description
+   * @param step Indent depth
+   *
+   * See ConfBase::write and overloads for use.
+   */
+  typedef void describe_type(std::ostream &, const std::string &, int);
+
+  /** @brief Describe a configuration item
+   * @param os Output stream
+   * @param description Description
+   * @param step Indent depth
+   *
+   * See ConfBase::write and overloads for use.
+   */
+  static void describe(std::ostream &os,
+                       const std::string &description,
+                       int step);
+
+  /** @brief No-op placeholder used instead of ConfBase::describe
+   * @param os Output stream
+   * @param description Description
+   * @param step Indent depth
+   *
+   * See ConfBase::write and overloads for use.
+   */
+  static void nodescribe(std::ostream &os,
+                         const std::string &description,
+                         int step);
+
   friend void test_quote();
   friend void test_quote_vector();
   friend void test_indent();
 };
 
-/** @brief Write a configuration node to a stream
- * @param os Output stream
- * @param c Configuration node
- * @return @p os
+/** @brief Type of map from host names to hosts
+ *
+ * @see Conf::hosts
  */
-inline std::ostream &operator<<(std::ostream &os, const ConfBase &c) {
-  c.write(os);
-  return os;
-}
+typedef std::map<std::string, Host *> hosts_type;
 
-/** @brief Type of map from host names to hosts */
-typedef std::map<std::string,Host *> hosts_type;
+/** @brief Type of map from store names to stores
+ *
+ * @see Conf::stores
+ */
+typedef std::map<std::string, Store *> stores_type;
 
-/** @brief Type of map from store names to stores */
-typedef std::map<std::string,Store *> stores_type;
-
-/** @brief Type of map from device names to devices */
-typedef std::map<std::string,Device *> devices_type;
+/** @brief Type of map from device names to devices
+ *
+ * @see Conf::devices
+ **/
+typedef std::map<std::string, Device *> devices_type;
 
 /** @brief Represents the entire configuration of rsbackup. */
 class Conf: public ConfBase {
 public:
-  /** @brief Construct an empty configuration */
-  Conf(): maxUsage(DEFAULT_MAX_USAGE),
-          maxFileUsage(DEFAULT_MAX_FILE_USAGE),
-          publicStores(false),
-          logs(DEFAULT_LOGS),
-          sshTimeout(DEFAULT_SSH_TIMEOUT),
-          keepPruneLogs(DEFAULT_KEEP_PRUNE_LOGS),
-          sendmail(DEFAULT_SENDMAIL),
-          unknownObjects(0),
-          logsRead(false),
-          devicesIdentified(false) { }
-
   /** @brief Map of host names to configuration */
   hosts_type hosts;
 
@@ -177,31 +202,31 @@ public:
    *
    * Corresponds to @c max-usage.  Currently not implemented.
    */
-  int maxUsage;
+  int maxUsage = DEFAULT_MAX_USAGE;
 
   /** @brief Maximum file usage
    *
    * Corresponds to @c max-file-usage.  Currently not implemented.
    */
-  int maxFileUsage;
+  int maxFileUsage = DEFAULT_MAX_FILE_USAGE;
 
   /** @brief Permit public stores */
-  bool publicStores;
+  bool publicStores = false;
 
   /** @brief Log directory */
-  std::string logs;
+  std::string logs = DEFAULT_LOGS;
 
   /** @brief Lockfile path */
   std::string lock;
 
-  /** @brief Timeout to pass to SSH */
-  int sshTimeout;
-
   /** @brief Age to keep pruning logs */
-  int keepPruneLogs;
+  int keepPruneLogs = DEFAULT_KEEP_PRUNE_LOGS;
+
+  /** @brief Age to report pruning logs */
+  int reportPruneLogs = DEFAULT_PRUNE_REPORT_AGE;
 
   /** @brief Path to @c sendmail */
-  std::string sendmail;
+  std::string sendmail = DEFAULT_SENDMAIL;
 
   /** @brief Pre-access hook */
   std::vector<std::string> preAccess;
@@ -209,12 +234,27 @@ public:
   /** @brief Post-access hook */
   std::vector<std::string> postAccess;
 
+  /** @brief Path to stylesheet for HTML report output
+   *
+   * If empty, a built-in one will be used.
+   */
+  std::string stylesheet;
+
+  /** @brief RGB triple for 'good' color code */
+  unsigned colorGood = COLOR_GOOD;
+
+  /** @brief RGB triple for 'bad' color code */
+  unsigned colorBad = COLOR_BAD;
+
   /** @brief Read the master configuration file
    * @throws IOError if a file cannot be read
    * @throws ConfigError if the contents of a file are malformed
    */
   void read();
 
+  /** @brief Validate a read configuration file */
+  void validate() const;
+
   /** @brief (De-)select one or more volumes
    * @param hostName Name of host containing volume to select
    * @param volumeName Name of volume to select or "*" for all
@@ -233,21 +273,21 @@ public:
 
   /** @brief Find a host by name
    * @param hostName Host to find
-   * @return Host, or NULL
+   * @return Host, or null pointer
    */
   Host *findHost(const std::string &hostName) const;
 
   /** @brief Find a volume by name
    * @param hostName Name of host containing volume
    * @param volumeName Name of volume to find
-   * @return Volume, or NULL
+   * @return Volume, or null pointer
    */
   Volume *findVolume(const std::string &hostName,
                      const std::string &volumeName) const;
 
   /** @brief Find a device by name
    * @param deviceName Name of device to find
-   * @return Device, or NULL
+   * @return Device, or null pointer
    */
   Device *findDevice(const std::string &deviceName) const;
 
@@ -259,7 +299,7 @@ public:
 
   /** @brief Identify devices
    * @param states Bitmap of store states to consider
-   * 
+   *
    * Safe to call multiple times.
    */
   void identifyDevices(int states);
@@ -280,10 +320,21 @@ public:
    *
    * Set by readState().
    */
-  int unknownObjects;
+  int unknownObjects = 0;
 
-  /** @brief Regexp used to parse logfiles names */
-  static Regexp logfileRegexp;
+  /** @brief Get the database access object
+   * @return Reference to database object
+   *
+   * Creates tables if they don't exist.
+   */
+  Database &getdb();
+
+  ConfBase *getParent() const override;
+
+  std::string what() const override;
+
+  void write(std::ostream &os, int step, bool verbose)
+    const override;
 
 private:
   /** @brief Read a single configuration file
@@ -305,36 +356,7 @@ private:
    * Otherwise the behavior is the same as @ref readOneFile().
    */
   void includeFile(const std::string &path);
-
-  /** @brief Split and parse a list represented as a string
-   * @param bits Destination for components of the string
-   * @param line String to parse
-   * @throws SyntaxError if @p line is malformed.
-   *
-   * Each component can be quoted or unquoted.
-   *
-   * Unquoted components are delimited by whitespace and cannot contain double
-   * quotes or backslashes.
-   *
-   * Quoted components are delimited by double quotes.  Within the quotes
-   * backslash can be used to escape the next character.
-   *
-   * The hash character can appear inside quotes or noninitially in an unquoted
-   * component, but otherwise introduces a comment which extends to the end of
-   * @p line.
-   */
-  static void split(std::vector<std::string> &bits, const std::string &line);
-
-  /** @brief Parse an integer
-   * @param s Decimal representation of integer
-   * @param min Minimum acceptable value
-   * @param max Maximum acceptable value
-   * @return Integer value
-   * @throws SyntaxError if the @p s doesn't represent an integer
-   * @throws SyntaxError if the integer value is out of range
-   */
-  static int parseInteger(const std::string &s,
-                          int min = INT_MIN, int max = INT_MAX);
+  friend struct IncludeDirective;
 
   /** @brief (De-)select all hosts
    * @param sense @c true to select all hosts, @c false to deselect them all
@@ -354,18 +376,34 @@ private:
   /** @brief Set to @c true when logfiles have been read
    * Set by @ref readState().
    */
-  bool logsRead;
+  bool logsRead = false;
 
   /** @brief Set to @c true when devices have been identified
    * Set by @ref identifyDevices().
    */
-  int devicesIdentified;
+  int devicesIdentified = false;
 
-  /** @brief Write this node to a stream
-   * @param os Output stream
-   * @param step Indent depth
+  /** @brief Database access object */
+  Database *db = nullptr;
+
+  /** @brief Create database tables */
+  void createTables();
+
+  /** @brief Validate and add a backup to a volume
+   * @param backup Populated backup
+   * @param hostName Host owning @p backup
+   * @param volumeName Volume owning @p backup
+   * @param forceWarn Force warnings on.
+   *
+   * Identifies the volume from the parameters, fills in @p Backup::volume, and
+   * adds the backup to the volume's list of backups.  If the backup belongs to
+   * an unknown device, host or volume, logs this but does not add it to
+   * anything.
    */
-  virtual void write(std::ostream &os, int step = 0) const;
+  void addBackup(Backup &backup,
+                 const std::string &hostName,
+                 const std::string &volumeName,
+                 bool forceWarn = false);
 };
 
 /** @brief Represents a backup device */
@@ -374,16 +412,16 @@ public:
   /** @brief Constructor
    * @param name_ Name of device
    */
-  Device(const std::string &name_): name(name_), store(NULL) {}
+  Device(const std::string &name_): name(name_) {}
 
   /** @brief Name of device */
   std::string name;
 
-  /** @brief Store for this device, or NULL
+  /** @brief Store for this device, or null pointer
    *
    * Set by Store::identify().
    */
-  Store *store;
+  Store *store = nullptr;
 
   /** @brief Validity test for device names
    * @param n Name of device
@@ -392,8 +430,10 @@ public:
   static bool valid(const std::string &n);
 };
 
-/** @brief Type of map from volume names to volumes */
-typedef std::map<std::string,Volume *> volumes_type;
+/** @brief Type of map from volume names to volumes
+ * @see Host::volumes
+ */
+typedef std::map<std::string, Volume *> volumes_type;
 
 /** @brief Represents a host */
 class Host: public ConfBase {
@@ -406,8 +446,7 @@ public:
     ConfBase(static_cast<ConfBase *>(parent_)),
     parent(parent_),
     name(name_),
-    hostname(name_),
-    alwaysUp(false) {
+    hostname(name_) {
     parent->addHost(this);
   }
 
@@ -427,7 +466,10 @@ public:
   std::string hostname;
 
   /** @brief True if host is expected to always be up */
-  bool alwaysUp;
+  bool alwaysUp = false;
+
+  /** @brief Priority of this host */
+  int priority = 0;
 
   /** @brief Unrecognized volume names found in logs
    *
@@ -454,7 +496,7 @@ public:
 
   /** @brief Find a volume by name
    * @param volumeName Name of volume to find
-   * @return Volume or NULL
+   * @return Volume or null pointer
    */
   Volume *findVolume(const std::string &volumeName) const;
 
@@ -479,40 +521,108 @@ public:
    */
   static bool valid(const std::string &n);
 
-private:
-  /** @brief Write this node to a stream
-   * @param os Output stream
-   * @param step Indent depth
+  /** @brief Invoke a command on the host and return its exit status
+   * @param capture Where to put capture stdout, or null pointer
+   * @param cmd Command to invoke
+   * @param ... Arguments to command, terminatd by a null pointer
+   * @return Exit status
    */
-  virtual void write(std::ostream &os, int step = 0) const;
+  int invoke(std::string *capture, const char *cmd, ...) const;
+
+  ConfBase *getParent() const override;
+
+  std::string what() const override;
+
+  void write(std::ostream &os, int step, bool verbose)
+    const override;
 };
 
+/** @brief Possible states of a backup */
+enum BackupStatus {
+  /** @brief %Backup status unknown */
+  UNKNOWN = 0,
+
+  /** @brief %Backup is underway */
+  UNDERWAY = 1,
+
+  /** @brief %Backup is complete */
+  COMPLETE = 2,
+
+  /** @brief %Backup failed */
+  FAILED = 3,
+
+  /** @brief Pruning is underway */
+  PRUNING = 4,
+
+  /** @brief Pruning is complete */
+  PRUNED = 5
+};
+
+/** @brief Names of @ref BackupStatus constants */
+extern const char *const backup_status_names[];
+
 /** @brief Represents the status of one backup */
 class Backup {
-public:
-  /** @brief Wait status
+  /** @brief Status of this backup
    *
-   * 0 means the backup succeeded.
+   * @see BackupStatus
    */
-  int rc;
+  int status = UNKNOWN;
 
-  /** @brief True if pruning has commenced */
-  bool pruning;
+public:
+  /** @brief Wait status from @c rsync
+   *
+   * This is a wait status as returned by @c waitpid and similar functions.  0
+   * means the backup succeeded.  It is meaningless if @ref Backup::status is
+   * @ref UNKNOWN or @ref UNDERWAY.
+   */
+  int rc = 0;
 
-  /** @brief Date of backup */
+  /** @brief Date of backup
+   *
+   * This reflects the day the backup was started.
+   */
   Date date;
 
+  /** @brief Id of backup
+   *
+   * In the current implementation these are days represented by YYYY-MM-DD
+   * format.  However, IDs are treated as opaque strings, so this could be
+   * changed.
+   *
+   * Note though that when importing logs from older versions of rsbackup (in
+   * Conf::readState) the filename is assumed to start YYYY-MM-DD and this is
+   * imported as both the time of the backup and its ID.
+   */
+  std::string id;
+
+  /** @brief Time of backup
+   *
+   * This reflects the time that the backup was started.
+   */
+  time_t time = 0;
+
+  /** @brief Time backup pruned
+   *
+   * The meaning of this member depends on the value of @ref Backup::status
+   *
+   * If the current status is @ref PRUNING then it reflects the time that it
+   * was decided to prune the backup.
+   *
+   * If the current status is @ref PRUNED then it reflects the time that the
+   * pruning operation completed.
+   *
+   * For any other status, the value is meaningless. */
+  time_t pruned = 0;
+
   /** @brief Device containing backup */
   std::string deviceName;
 
   /** @brief Log contents */
-  std::vector<std::string> contents;
+  std::string contents;
 
   /** @brief Volume backed up */
-  Volume *volume;
-
-  /** @brief Why this backup was pruned */
-  std::string whyPruned;
+  Volume *volume = nullptr;
 
   /** @brief Ordering on backups
    * @param that Other backup
@@ -530,14 +640,41 @@ public:
   /** @brief Return path to backup */
   std::string backupPath() const;
 
-  /** @brief Return path to logfile */
-  std::string logPath() const;
-
   /** @brief Return containing device
    *
-   * TODO could this be NULL if device has been retired?
+   * @todo could this be null pointer if device has been retired?
    */
   Device *getDevice() const;
+
+  /** @brief Insert this backup into the database
+   * @param db Database to update
+   * @param replace Replace existing row if present
+   */
+  void insert(Database &db,
+              bool replace = false) const;
+
+  /** @brief Update this backup in the database
+   * @param db Database to update
+   */
+  void update(Database &db) const;
+
+  /** @brief Remove this backup from the database
+   * @param db Database to update
+   */
+  void remove(Database &db) const;
+
+  /** @brief Retrieve status of this backup
+   * @return Status (see @ref BackupStatus)
+   */
+  inline int getStatus() const {
+    return status;
+  }
+
+  /** @brief Set the status of this backup
+   * @param n New status (see @ref BackupStatus)
+   *
+   * Calls Volume::calculate if necessary. */
+  void setStatus(int n);
 };
 
 /** @brief Comparison for backup pointers */
@@ -552,7 +689,9 @@ struct compare_backup {
   }
 };
 
-/** @brief Type of an ordered set of backups */
+/** @brief Type of an ordered set of backups
+ * @see Volume::backups
+ */
 typedef std::set<Backup *,compare_backup> backups_type;
 
 /** @brief Represents a single volume (usually, filesystem) to back up */
@@ -568,10 +707,7 @@ public:
          const std::string &path_): ConfBase(static_cast<ConfBase *>(parent_)),
                                     parent(parent_),
                                     name(name_),
-                                    path(path_),
-                                    traverse(false),
-                                    completed(0),
-                                    isSelected(false) {
+                                    path(path_) {
     parent->addVolume(this);
   }
 
@@ -588,11 +724,14 @@ public:
   std::vector<std::string> exclude;
 
   /** @brief Traverse mount points if true */
-  bool traverse;
+  bool traverse = false;
 
   /** @brief File to check before backing up */
   std::string checkFile;
 
+  /** @brief Check that root path is a mount point before backing up */
+  bool checkMounted = false;
+
   /** @brief Return true if volume is selected */
   bool selected() const { return isSelected; }
 
@@ -618,23 +757,17 @@ public:
   /** @brief Per-device information about this volume */
   struct PerDevice {
     /** @brief Number of backups of volume on device */
-    int count;
-
-    /** @brief Number of backups of volume to be removed from device */
-    int toBeRemoved;
+    int count = 0;
 
     /** @brief Oldest backup of volume on device */
     Date oldest;
 
     /** @brief Newest backup of volume on device */
     Date newest;
-
-    /** @brief Construct per-device information */
-    PerDevice(): count(0), toBeRemoved(0) {}
   };
 
   /** @brief Number of completed backups */
-  int completed;
+  int completed = 0;
 
   /** @brief Date of oldest backup (on any device) */
   Date oldest;
@@ -643,7 +776,7 @@ public:
   Date newest;
 
   /** @brief Type for @ref perDevice */
-  typedef std::map<std::string,PerDevice> perdevice_type;
+  typedef std::map<std::string, PerDevice> perdevice_type;
 
   /** @brief Map of device names to per-device information */
   perdevice_type perDevice;
@@ -655,20 +788,27 @@ public:
   bool removeBackup(const Backup *backup);
 
   /** @brief Find the most recent backup
-   * @param device If not NULL, only consider backups from this device
-   * @return Most recent backup or NULL
+   * @param device If not null pointer, only consider backups from this device
+   * @return Most recent backup or null pointer
    */
-  const Backup *mostRecentBackup(const Device *device = NULL) const;
+  const Backup *mostRecentBackup(const Device *device = nullptr) const;
 
   /** @brief Find the most recent failedbackup
-   * @param device If not NULL, only consider backups from this device
-   * @return Most recent failed backup or NULL
+   * @param device If not null pointer, only consider backups from this device
+   * @return Most recent failed backup or null pointer
    */
-  const Backup *mostRecentFailedBackup(const Device *device = NULL) const;
+  const Backup *mostRecentFailedBackup(const Device *device = nullptr) const;
+
+  ConfBase *getParent() const override;
+
+  std::string what() const override;
+
+  void write(std::ostream &os, int step, bool verbose)
+    const override;
 
 private:
   /** @brief Set to @c true if this volume is selected */
-  bool isSelected;
+  bool isSelected = false;
 
   /** @brief Recalculate statistics
    *
@@ -684,11 +824,7 @@ private:
    */
   void calculate();
 
-  /** @brief Write this node to a stream
-   * @param os Output stream
-   * @param step Indent depth
-   */
-  virtual void write(std::ostream &os, int step = 0) const;
+  friend void Backup::setStatus(int);
 };
 
 inline Device *Backup::getDevice() const {
