@@ -1,5 +1,5 @@
 // -*-C++-*-
-// Copyright © 2011, 2012 Richard Kettlewell.
+// Copyright © 2011, 2012, 2014, 2015 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -41,10 +41,25 @@
 #define DEFAULT_MAX_AGE 3
 
 /** @brief Default minimum backups */
-#define DEFAULT_MIN_BACKUPS 1
+#define DEFAULT_MIN_BACKUPS "1"
 
 /** @brief Default pruning age */
-#define DEFAULT_PRUNE_AGE 366
+#define DEFAULT_PRUNE_AGE "366"
+
+/** @brief Default pruning age */
+#define DEFAULT_DECAY_START "1"
+
+/** @brief Default decay window size */
+#define DEFAULT_DECAY_WINDOW "1"
+
+/** @brief Default decay scale */
+#define DEFAULT_DECAY_SCALE "2"
+
+/** @brief Default pruning policy */
+#define DEFAULT_PRUNE_POLICY "age"
+
+/** @brief Default age for pruning logs in report */
+#define DEFAULT_PRUNE_REPORT_AGE 3
 
 /** @brief Default maximum disk usage */
 #define DEFAULT_MAX_USAGE 80
@@ -90,11 +105,14 @@
 #define DEVICE_VALID ALPHA DIGIT "_."
 
 /** @brief Valid characters in a host name */
-#define HOST_VALID ALPHA DIGIT "."
+#define HOST_VALID ALPHA DIGIT ".-"
 
 /** @brief Valid characters in a volume name */
 #define VOLUME_VALID ALPHA DIGIT "_."
 
+/** @brief Valid characters in a prune-parameter name */
+#define PRUNE_PARAMETER_VALID ALPHA DIGIT "-"
+
 /** @brief Path separator */
 #define PATH_SEP "/"
 
