@@ -1,4 +1,4 @@
-// Copyright © 2011-2013 Richard Kettlewell.
+// Copyright © 2011-2015 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,66 +17,91 @@
 #include "Document.h"
 #include "Conf.h"
 #include "Command.h"
-#include "Regexp.h"
 #include "IO.h"
+#include "Database.h"
+#include "Report.h"
+#include "Utils.h"
 #include <cmath>
 #include <cstdlib>
 #include <stdexcept>
+#include <boost/range/adaptor/reversed.hpp>
 
 // Split up a color into RGB components
-static void unpackColor(unsigned color, int rgb[3]) {
+void Report::unpackColor(unsigned color, int rgb[3]) {
   rgb[0] = (color >> 16) & 255;
   rgb[1] = (color >> 8) & 255;
   rgb[2] = color & 255;
 }
 
 // Pack a color from RGB components
-static unsigned packColor(const int rgb[3]) {
+unsigned Report::packColor(const int rgb[3]) {
   return rgb[0] * 65536 + rgb[1] * 256 + rgb[2];
 }
 
 // Pick a color as a (clamped) linear combination of two endpoints
-static unsigned pickColor(unsigned zero, unsigned one, double param) {
+unsigned Report::pickColor(unsigned zero, unsigned one, double param) {
   int zeroRgb[3], oneRgb[3], resultRgb[3];
   unpackColor(zero, zeroRgb);
   unpackColor(one, oneRgb);
-  if(param < 0) param = 0;
-  if(param > 1) param = 1;
+  if(param < 0)
+    param = 0;
+  if(param > 1)
+    param = 1;
   resultRgb[0] = zeroRgb[0] * (1 - param) + oneRgb[0] * param;
   resultRgb[1] = zeroRgb[1] * (1 - param) + oneRgb[1] * param;
   resultRgb[2] = zeroRgb[2] * (1 - param) + oneRgb[2] * param;
   return packColor(resultRgb);
 }
 
-// Generate the list of unknown devices, hosts and volumes
-static void reportUnknown(Document &d) {
+// Generate the list of warnings
+void Report::reportWarnings() {
+  char buffer[1024];
   Document::List *l = new Document::List();
-  for(std::set<std::string>::iterator it = config.unknownDevices.begin();
-      it != config.unknownDevices.end();
-      ++it) {
-    l->entry("Unknown device " + *it);
+  for(auto &d: config.unknownDevices) {
+    l->entry("Unknown device " + d);
+    ++devices_unknown;
   }
-  for(std::set<std::string>::iterator it = config.unknownHosts.begin();
-      it != config.unknownHosts.end();
-      ++it) {
-    l->entry("Unknown host " + *it);
+  for(auto &h: config.unknownHosts) {
+    l->entry("Unknown host " + h);
+    ++hosts_unknown;
   }
-  for(hosts_type::iterator hostsIterator = config.hosts.begin();
-      hostsIterator != config.hosts.end();
-      ++hostsIterator) {
-    const std::string &hostName = hostsIterator->first;
-    Host *host = hostsIterator->second;
-    for(std::set<std::string>::iterator it = host->unknownVolumes.begin();
-        it != host->unknownVolumes.end();
-        ++it) {
-      l->entry("Unknown volume " + *it + " on host " + hostName);
+  for(auto &h: config.hosts) {
+    const std::string &hostName = h.first;
+    Host *host = h.second;
+    for(auto &v: host->unknownVolumes) {
+      l->entry("Unknown volume " + v + " on host " + hostName);
+      ++volumes_unknown;
     }
   }
-  d.append(l);
+  if(backups_missing) {
+    snprintf(buffer, sizeof buffer, "WARNING: %d volumes have no backups.",
+             backups_missing);
+    l->entry(buffer);
+  }
+  if(backups_partial) {
+    snprintf(buffer, sizeof buffer, "WARNING: %d volumes are not fully backed up.",
+             backups_partial);
+    l->entry(buffer);
+  }
+  if(backups_out_of_date) {
+    snprintf(buffer, sizeof buffer, "WARNING: %d volumes are out of date.",
+             backups_out_of_date);
+    l->entry(buffer);
+  }
+  if(backups_failed) {
+    snprintf(buffer, sizeof buffer, "WARNING: %d volumes failed latest backup.",
+             backups_failed);
+    l->entry(buffer);
+  }
+  if(l->nodes.size() > 0) {
+    d.heading("Warnings", 2);
+    d.append(l);
+  } else
+    delete l;
 }
 
 // Generate the summary table
-static Document::Table *reportSummary() {
+Document::Table *Report::reportSummary() {
   Document::Table *t = new Document::Table();
 
   t->addHeadingCell(new Document::Cell("Host", 1, 3));
@@ -86,39 +111,27 @@ static Document::Table *reportSummary() {
   t->addHeadingCell(new Document::Cell("Devices", 2 * config.devices.size(), 1));
   t->newRow();
 
-  for(devices_type::const_iterator it = config.devices.begin();
-      it != config.devices.end();
-      ++it) {
-    const Device *device = it->second;
-    t->addHeadingCell(new Document::Cell(device->name, 2, 1));
-  }
+  for(auto &d: config.devices)
+    t->addHeadingCell(new Document::Cell(d.second->name, 2, 1));
   t->newRow();
 
-  for(devices_type::const_iterator it = config.devices.begin();
-      it != config.devices.end();
-      ++it) {
+  for(auto attribute((unused)) &d: config.devices) {
     t->addHeadingCell(new Document::Cell("Newest"));
     t->addHeadingCell(new Document::Cell("Count"));
   }
   t->newRow();
 
-  for(hosts_type::iterator ith = config.hosts.begin();
-      ith != config.hosts.end();
-      ++ith) {
-    Host *host = ith->second;
+  for(auto &h: config.hosts) {
+    Host *host = h.second;
     t->addCell(new Document::Cell(host->name, 1, host->volumes.size()))
       ->style = "host";
-    for(volumes_type::iterator itv = host->volumes.begin();
-        itv != host->volumes.end();
-        ++itv) {
-      Volume *volume = itv->second;
+    for(auto &v: host->volumes) {
+      Volume *volume = v.second;
       // See if every device has a backup
       bool missingDevice = false;
-      for(devices_type::iterator itd = config.devices.begin();
-          itd != config.devices.end();
-          ++itd) {
-        Device *device = itd->second;
-        if(volume->perDevice.find(device->name) == volume->perDevice.end())
+      for(auto &d: config.devices) {
+        Device *device = d.second;
+        if(!contains(volume->perDevice, device->name))
           missingDevice = true;
       }
       t->addCell(new Document::Cell(volume->name))
@@ -128,27 +141,49 @@ static Document::Table *reportSummary() {
                                     : "none"));
       t->addCell(new Document::Cell(new Document::String(volume->completed)))
         ->style = missingDevice ? "bad" : "good";
-      for(devices_type::const_iterator it = config.devices.begin();
-          it != config.devices.end();
-          ++it) {
-        const Device *device = it->second;
+      bool out_of_date = true;
+      size_t devices_used = 0;
+      for(auto &d: config.devices) {
+        const Device *device = d.second;
         Volume::PerDevice &perDevice = volume->perDevice[device->name];
         if(perDevice.count) {
+          // At least one successful backups
           Document::Cell *c
             = t->addCell(new Document::Cell(perDevice.newest.toString()));
           int newestAge = Date::today() - perDevice.newest;
           if(newestAge <= volume->maxAge) {
             double param = (pow(2, (double)newestAge / volume->maxAge) - 1) / 2.0;
-            c->bgcolor = pickColor(COLOR_GOOD, COLOR_BAD, param);
+            c->bgcolor = pickColor(config.colorGood, config.colorBad, param);
+            out_of_date = false;
           } else {
             c->style = "bad";
           }
-        } else
+          ++devices_used;
+        } else {
+          // No succesful backups!
           t->addCell(new Document::Cell("none"))
             ->style = "bad";
+        }
         t->addCell(new Document::Cell(new Document::String(perDevice.count)))
           ->style = perDevice.count ? "good" : "bad";
+        // Look for the most recent attempt at this device
+        const Backup *most_recent_backup = nullptr;
+        for(const Backup *b: boost::adaptors::reverse(volume->backups)) {
+          if(b->getStatus() == COMPLETE && b->deviceName == device->name) {
+            most_recent_backup = b;
+            break;
+          }
+        }
+        if(most_recent_backup && most_recent_backup->rc != 0)
+          ++backups_failed;
       }
+      if(devices_used < config.devices.size()) {
+        if(devices_used == 0)
+          ++backups_missing;
+        else
+          ++backups_partial;
+      } else if(out_of_date)
+        ++backups_out_of_date;
       t->newRow();
     }
   }
@@ -157,7 +192,7 @@ static Document::Table *reportSummary() {
 }
 
 // Return true if this is a suitable log for the report
-static bool suitableLog(const Volume *volume, const Backup *backup) {
+bool Report::suitableLog(const Volume *volume, const Backup *backup) {
   // Empty logs are never shown.
   if(!backup->contents.size())
     return false;
@@ -184,18 +219,14 @@ static bool suitableLog(const Volume *volume, const Backup *backup) {
 }
 
 // Generate the report of backup logfiles for a volume
-static void reportLogs(Document &d,
-                       Volume *volume) {
-  Document::LinearContainer *lc = NULL;
+void Report::reportLogs(const Volume *volume) {
+  Document::LinearContainer *lc = nullptr;
   Host *host = volume->parent;
   // Backups for a volume are ordered primarily by date and secondarily by
   // device.  The most recent backups are the most interesting so they are
   // displayed in reverse.
   std::set<std::string> devicesSeen;
-  for(backups_type::reverse_iterator backupsIterator = volume->backups.rbegin();
-      backupsIterator != volume->backups.rend();
-      ++backupsIterator) {
-    const Backup *backup = *backupsIterator;
+  for(const Backup *backup: boost::adaptors::reverse(volume->backups)) {
     // Only include logs of failed backups
     if(suitableLog(volume, backup)) {
       if(!lc) {
@@ -206,12 +237,14 @@ static void reportLogs(Document &d,
         lc->style = "volume";
         d.append(lc);
       }
-      Document::Heading *heading = 
+      Document::Heading *heading =
         new Document::Heading(backup->date.toString()
                               + " device " + backup->deviceName
-                              + " (" + backup->logPath() + ")",
+                              + " volume "
+                                 + backup->volume->parent->name
+                                 + ":" + backup->volume->name,
                               4);
-      if(devicesSeen.find(backup->deviceName) == devicesSeen.end())
+      if(!contains(devicesSeen, backup->deviceName))
         heading->style = "recent";
       lc->append(heading);
       Document::Verbatim *v = new Document::Verbatim();
@@ -223,75 +256,96 @@ static void reportLogs(Document &d,
   }
 }
 
-// Generate the rpeort of backup logfiles for everything
-static void reportLogs(Document &d) {
+// Generate the report of backup logfiles for everything
+void Report::reportLogs() {
   // Sort by host/volume first, then date, device *last*
-  for(hosts_type::iterator hostsIterator = config.hosts.begin();
-      hostsIterator != config.hosts.end();
-      ++hostsIterator) {
-    Host *host = hostsIterator->second;
-    for(volumes_type::iterator volumesIterator = host->volumes.begin();
-        volumesIterator != host->volumes.end();
-        ++volumesIterator) {
-      Volume *volume = volumesIterator->second;
-      reportLogs(d, volume);
+  for(auto &h: config.hosts) {
+    Host *host = h.second;
+    for(auto &v: host->volumes) {
+      Volume *volume = v.second;
+      reportLogs(volume);
     }
   }
 }
 
 // Generate the report of pruning logfiles
-static void reportPruneLogs(Document &d) {
-  std::map<Date,std::string> pruneLogs;
-  // Retrieve pruning logs.
-  Regexp r("^prune-([0-9]+-[0-9]+-[0-9]+)\\.log$");
-  Directory dir;
-  dir.open(config.logs);
-  std::string f;
-  while(dir.get(f)) {
-    if(!r.matches(f))
-      continue;
-    Date date = r.sub(1);
-    pruneLogs[date] = config.logs + PATH_SEP + f;
-  }
-  // Display them, most recent first
-  for(std::map<Date,std::string>::reverse_iterator pruneLogsIterator = pruneLogs.rbegin();
-      pruneLogsIterator != pruneLogs.rend();
-      ++pruneLogsIterator) {
-    const std::string &path = pruneLogsIterator->second;
-    Document::Verbatim *v = d.verbatim();
-    v->style = "log";
-    IO logFile;
-    logFile.open(path, "r");
-    std::string line;
-    while(logFile.readline(line)) {
-      v->append(line);
-      v->append("\n");
-    }
+Document::Node *Report::reportPruneLogs() {
+  Document::Table *t = new Document::Table();
+
+  t->addHeadingCell(new Document::Cell("Created", 1, 1));
+  t->addHeadingCell(new Document::Cell("Pruned", 1, 1));
+  t->addHeadingCell(new Document::Cell("Host", 1, 1));
+  t->addHeadingCell(new Document::Cell("Volume", 1, 1));
+  t->addHeadingCell(new Document::Cell("Device", 1, 1));
+  t->addHeadingCell(new Document::Cell("Reason", 1, 1));
+  t->newRow();
+
+  const int64_t cutoff = Date::now() - 86400 * config.reportPruneLogs;
+  Database::Statement stmt(config.getdb(),
+                           "SELECT host,volume,device,time,pruned,log"
+                           " FROM backup"
+                           " WHERE (status=? OR status=?) AND pruned >= ?"
+                           " ORDER BY pruned DESC",
+                           SQL_INT, PRUNING,
+                           SQL_INT, PRUNED,
+                           SQL_INT64, cutoff,
+                           SQL_END);
+  while(stmt.next()) {
+    Backup backup;
+    char timestr[64];
+    std::string hostName = stmt.get_string(0);
+    std::string volumeName = stmt.get_string(1);
+    std::string deviceName = stmt.get_string(2);
+    time_t when = stmt.get_int64(3);
+    time_t pruned = stmt.get_int64(4);
+    std::string reason = stmt.get_blob(5);
+
+    strftime(timestr, sizeof timestr, "%Y-%m-%d", localtime(&when));
+    t->addCell(new Document::Cell(timestr));
+    strftime(timestr, sizeof timestr, "%Y-%m-%d", localtime(&pruned));
+    t->addCell(new Document::Cell(timestr));
+    t->addCell(new Document::Cell(hostName));
+    t->addCell(new Document::Cell(volumeName));
+    t->addCell(new Document::Cell(deviceName));
+    t->addCell(new Document::Cell(reason));
+
+    t->newRow();
   }
+
+  return t;
 }
 
 // Generate the full report
-void generateReport(Document &d) {
+void Report::generate() {
+  backups_missing = 0;
+  backups_partial = 0;
+  backups_out_of_date = 0;
+  backups_failed = 0;
+  devices_unknown = 0;
+  hosts_unknown = 0;
+  volumes_unknown = 0;
+
   d.title = "Backup report (" + Date::today().toString() + ")";
   d.heading(d.title);
 
+  /* Generating the summary has a side effect of filling in the summary
+   * counters */
+  Document::Table *report = reportSummary();
+
   // Unknown objects ----------------------------------------------------------
-  if(config.unknownObjects) {
-    d.heading("Warnings", 2);
-    reportUnknown(d);
-  }
+  reportWarnings();
 
   // Summary table ------------------------------------------------------------
   d.heading("Summary", 2);
-  d.append(reportSummary());
+  d.append(report);
 
   // Logfiles -----------------------------------------------------------------
   d.heading("Logfiles", 2);
-  reportLogs(d);
+  reportLogs();
 
   // Prune logs ---------------------------------------------------------------
   d.heading("Pruning logs", 3);         // TODO anchor
-  reportPruneLogs(d);
+  d.append(reportPruneLogs());
 
   // Generation time ----------------------------------------------------------
   Document::Paragraph *p = d.para("Generated ");
@@ -302,5 +356,4 @@ void generateReport(Document &d) {
     time(&now);
     p->append(new Document::String(ctime(&now)));
   }
-
 }
