@@ -20,10 +20,12 @@
 #include "Utils.h"
 #include "DeviceAccess.h"
 #include <cerrno>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 // Identify the device on this store, if any
 void Store::identify() {
-  IO *f = NULL;
+  IO *f = nullptr;
   try {
     struct stat sb;
 
@@ -40,7 +42,7 @@ void Store::identify() {
     if(!f->readline(deviceName))
       throw BadStore("store '" + path + "' has a malformed device-id");
     // See if it exists
-    devices_type::iterator devices_iterator = config.devices.find(deviceName);
+    auto devices_iterator = config.devices.find(deviceName);
     if(devices_iterator == config.devices.end())
       throw BadStore("store '" + path
                      + "' has unknown device-id '" + deviceName + "'");
