@@ -1,4 +1,4 @@
-// Copyright © 2011-14 Richard Kettlewell.
+// Copyright © 2011-15 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,15 +25,17 @@
 #include "Subprocess.h"
 #include "DeviceAccess.h"
 #include "Utils.h"
+#include "Report.h"
 #include <cstdio>
 #include <cstdlib>
 #include <cerrno>
 #include <sstream>
 #include <iostream>
+#include <iomanip>
 
 int main(int argc, char **argv) {
   try {
-    if(setlocale(LC_CTYPE, "") == NULL)
+    if(setlocale(LC_CTYPE, "") == nullptr)
       throw std::runtime_error(std::string("setlocale: ") + strerror(errno));
 
     // Parse command line
@@ -42,22 +44,23 @@ int main(int argc, char **argv) {
     // Read configuration
     config.read();
 
+    // Validate configuration
+    config.validate();
+
     // Dump configuration
     if(command.dumpConfig) {
-      std::cout << config;
+      config.write(std::cout, 0, command.verbose);
       exit(0);
     }
 
     // Override stores
     if(command.stores.size() != 0) {
-      for(stores_type::iterator it = config.stores.begin();
-          it != config.stores.end();
-          ++it)
-        it->second->state = Store::Disabled;
-      for(size_t n = 0; n < command.stores.size(); ++n) {
-        stores_type::iterator it = config.stores.find(command.stores[n]);
+      for(auto &s: config.stores)
+        s.second->state = Store::Disabled;
+      for(auto &s: command.stores) {
+        auto it = config.stores.find(s);
         if(it == config.stores.end())
-          config.stores[command.stores[n]] = new Store(command.stores[n]);
+          config.stores[s] = new Store(s);
         else
           it->second->state = Store::Enabled;
       }
@@ -105,8 +108,24 @@ int main(int argc, char **argv) {
       config.readState();
 
       Document d;
-      d.htmlStyleSheet = stylesheet;
-      generateReport(d);
+      if(config.stylesheet.size()) {
+        IO ssf;
+        ssf.open(config.stylesheet, "r");
+        ssf.readall(d.htmlStyleSheet);
+      } else
+        d.htmlStyleSheet = stylesheet;
+      // Include user colors in the stylesheet
+      std::stringstream ss;
+      ss << std::hex;
+      ss << "td.bad { background-color: #"
+         << std::setw(6) << std::setfill('0') << config.colorBad << " }\n";
+      ss << "td.good { background-color: #"
+         << std::setw(6) << std::setfill('0')  << config.colorGood << " }\n";
+      ss << "span.bad { color: #"
+         << std::setw(6) << std::setfill('0')  << config.colorBad << " }\n";
+      d.htmlStyleSheet += ss.str();
+      Report report(d);
+      report.generate();
       std::stringstream htmlStream, textStream;
       if(command.html || command.email)
         d.renderHtml(htmlStream);
@@ -135,20 +154,36 @@ int main(int argc, char **argv) {
       if(command.email) {
         Email e;
         e.addTo(*command.email);
-        e.setSubject(d.title);
-        e.setType("multipart/alternative; boundary="MIME_BOUNDARY);
+        std::stringstream subject;
+        subject << d.title;
+        if(report.backups_missing)
+          subject << " missing:" << report.backups_missing;
+        if(report.backups_partial)
+          subject << " partial:" << report.backups_partial;
+        if(report.backups_out_of_date)
+          subject << " stale:" << report.backups_out_of_date;
+        if(report.backups_failed)
+          subject << " failed:" << report.backups_failed;
+        if(report.devices_unknown
+           || report.hosts_unknown
+           || report.volumes_unknown)
+          subject << " unknown:" << (report.devices_unknown
+                                     + report.hosts_unknown
+                                     + report.volumes_unknown);
+        e.setSubject(subject.str());
+        e.setType("multipart/alternative; boundary=" MIME_BOUNDARY);
         std::stringstream body;
-        body << "--"MIME_BOUNDARY"\n";
+        body << "--" MIME_BOUNDARY "\n";
         body << "Content-Type: text/plain\n";
         body << "\n";
         body << textStream.str();
         body << "\n";
-        body << "--"MIME_BOUNDARY"\n";
+        body << "--" MIME_BOUNDARY "\n";
         body << "Content-Type: text/html\n";
         body << "\n";
         body << htmlStream.str();
         body << "\n";
-        body << "--"MIME_BOUNDARY"--\n";
+        body << "--" MIME_BOUNDARY "--\n";
         e.setContent(body.str());
         e.send();
       }
@@ -156,6 +191,10 @@ int main(int argc, char **argv) {
     if(errors && command.verbose)
       warning("%d errors detected", errors);
     IO::out.close();
+  } catch(Error &e) {
+    error("%s", e.what());
+    if(command.debug)
+      e.trace(stderr);
   } catch(std::runtime_error &e) {
     error("%s", e.what());
   }
