@@ -16,6 +16,10 @@
 export WORKSPACE="${PWD}/w-${0##*/}"
 RSBACKUP="${VALGRIND} ${PWD}/../src/rsbackup --config ${WORKSPACE}/config ${VERBOSE}"
 
+PRUNE_POLICY="${PRUNE_POLICY:-age}"
+PRUNE_AGE="${PRUNE_AGE:-prune-age}"
+MIN_BACKUPS="${MIN_BACKUPS:-min-backups}"
+
 setup() {
   echo
   echo "* ==== $0 ===="
@@ -33,29 +37,34 @@ setup() {
   echo "store ${WORKSPACE}/store2" >>${WORKSPACE}/config
   echo "device device2" >> ${WORKSPACE}/config
 
-  echo "public" >> ${WORKSPACE}/config
+  echo "public true" >> ${WORKSPACE}/config
 
   echo "pre-access-hook ${srcdir:-.}/hook" >> ${WORKSPACE}/config
   echo "post-access-hook ${srcdir:-.}/hook" >> ${WORKSPACE}/config
 
+  echo "keep-prune-logs 1" >> ${WORKSPACE}/config
+  echo "prune-policy ${PRUNE_POLICY}" >> ${WORKSPACE}/config
+  [ -n "$PRUNE_PATH" ] && echo "prune-parameter path ${PRUNE_PATH}" >> ${WORKSPACE}/config
+  [ -n "$DECAY_LIMIT" ] && echo "prune-parameter decay-limit ${DECAY_LIMIT}" >> ${WORKSPACE}/config
+
   mkdir ${WORKSPACE}/logs
   echo "logs ${WORKSPACE}/logs" >> ${WORKSPACE}/config
   echo "lock ${WORKSPACE}/lock" >> ${WORKSPACE}/config
 
   echo "host host1" >> ${WORKSPACE}/config
   echo "  hostname localhost" >> ${WORKSPACE}/config
-  echo "  prune-age 2" >> ${WORKSPACE}/config
+  [ "${PRUNE_AGE}" != none ] && echo "  ${PRUNE_AGE} 2" >> ${WORKSPACE}/config
   echo "  volume volume1 ${WORKSPACE}/volume1" >> ${WORKSPACE}/config
-  echo "    min-backups 1" >> ${WORKSPACE}/config
+  [ "${MIN_BACKUPS}" != none ] && echo "    ${MIN_BACKUPS} 1" >> ${WORKSPACE}/config
   echo "    pre-backup-hook ${srcdir:-.}/hook" >> ${WORKSPACE}/config
   echo "    post-backup-hook ${srcdir:-.}/hook" >> ${WORKSPACE}/config
   echo "    check-file file1" >> ${WORKSPACE}/config
   echo "  volume volume2 ${WORKSPACE}/volume2" >> ${WORKSPACE}/config
-  echo "    min-backups 2" >> ${WORKSPACE}/config
+  [ "${MIN_BACKUPS}" != none ] && echo "    ${MIN_BACKUPS} 2" >> ${WORKSPACE}/config
   echo "  volume volume3 ${WORKSPACE}/volume3" >> ${WORKSPACE}/config
-  echo "    min-backups 2" >> ${WORKSPACE}/config
+  [ "${MIN_BACKUPS}" != none ] && echo "    ${MIN_BACKUPS} 2" >> ${WORKSPACE}/config
   echo "    devices *2" >> ${WORKSPACE}/config
-  
+
   mkdir ${WORKSPACE}/volume1
   echo one > ${WORKSPACE}/volume1/file1
   mkdir ${WORKSPACE}/volume1/dir1
@@ -108,7 +117,11 @@ absent() {
 s() {
   echo ">" "$@" "#" ${RSBACKUP_TODAY} >&2
   if [ -z "$STDERR" ]; then
-    "$@"
+    if [ -z "${RSBACKUP_TODAY}" ]; then
+      RUN="${RUN}" "$@"
+    else
+      RUN="${RUN}" RSBACKUP_TODAY="${RSBACKUP_TODAY}" "$@"
+    fi
   else
     if "$@" 2> "$STDERR"; then
       cat "$STDERR" >&2
